@@ -39,6 +39,7 @@
 #include "common.h"
 #include "dhcpcd.h"
 #include "if.h"
+#include "if-options.h"
 #include "ipv4.h"
 #include "ipv4ll.h"
 #include "ipv6.h"
@@ -139,7 +140,19 @@ rt_compare_os(__unused void *context, const void *node1, const void *node2)
 }
 
 static int
-rt_compare_proto(__unused void *context, const void *node1, const void *node2)
+rt_compare_list(__unused void *context, const void *node1, const void *node2)
+{
+	const struct rt *rt1 = node1, *rt2 = node2;
+
+	if (rt1->rt_order > rt2->rt_order)
+		return 1;
+	if (rt1->rt_order < rt2->rt_order)
+		return -1;
+	return 0;
+}
+
+static int
+rt_compare_proto(void *context, const void *node1, const void *node2)
 {
 	const struct rt *rt1 = node1, *rt2 = node2;
 	int c;
@@ -161,11 +174,7 @@ rt_compare_proto(__unused void *context, const void *node1, const void *node2)
 		return c;
 
 	/* Finally the order in which the route was given to us. */
-	if (rt1->rt_order > rt2->rt_order)
-		return 1;
-	if (rt1->rt_order < rt2->rt_order)
-		return -1;
-	return 0;
+	return rt_compare_list(context, rt1, rt2);
 }
 
 static const rb_tree_ops_t rt_compare_os_ops = {
@@ -175,6 +184,13 @@ static const rb_tree_ops_t rt_compare_os_ops = {
 	.rbto_context = NULL
 };
 
+const rb_tree_ops_t rt_compare_list_ops = {
+	.rbto_compare_nodes = rt_compare_list,
+	.rbto_compare_key = rt_compare_list,
+	.rbto_node_offset = offsetof(struct rt, rt_tree),
+	.rbto_context = NULL
+};
+
 const rb_tree_ops_t rt_compare_proto_ops = {
 	.rbto_compare_nodes = rt_compare_proto,
 	.rbto_compare_key = rt_compare_proto,
@@ -670,6 +686,7 @@ rt_build(struct dhcpcd_ctx *ctx, int af)
 	rb_tree_init(&kroutes, &rt_compare_os_ops);
 	if_initrt(ctx, &kroutes, af);
 	ctx->rt_order = 0;
+	ctx->options |= DHCPCD_RTBUILD;
 
 	switch (af) {
 #ifdef INET
